using System.Xml;
using System.Xml.Linq;
using System.Xml.Schema;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;

namespace FitConnect.Validation;

public class SubmissionDataValidation {
    public SubmissionDataValidation() {
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="data"></param>
    /// <param name="schema"></param>
    /// <param name="type"></param>
    public async Task<bool> ValidateSubmission(string data, string schema,
        string type = System.Net.Mime.MediaTypeNames.Application.Json) {
        if (string.IsNullOrWhiteSpace(data)) {
            throw new Exception("Data is required");
        }


        if (string.IsNullOrWhiteSpace(schema)) {
            throw new Exception("Schema is required");
        }

        if (schema.Contains("json") || type == System.Net.Mime.MediaTypeNames.Application.Json) {
            return await ValidationJsonData(data, schema);
        }
        else if (schema.Contains("xml") || type == System.Net.Mime.MediaTypeNames.Application.Xml) {
            return await ValidateXmlData(data, schema);
        }
        else {
            throw new Exception("Invalid schema");
        }
    }

    private async Task<bool> ValidationJsonData(string data, string schema) {
        var schemaJson = JSchema.Parse(schema);
        var dataJson = JObject.Parse(data);
        var isValid = dataJson.IsValid(schemaJson, out IList<string> errors);
        if (!isValid) {
            throw new Exception($"Invalid data: {string.Join(", ", errors)}");
        }

        return isValid;
    }

    private async Task<bool> ValidateXmlData(string data, string schema) {
        var schemaSet = new XmlSchemaSet();
        schemaSet.Add("", XmlReader.Create(new StringReader(schema)));
        var doc = XDocument.Parse(data);
        doc.Validate(schemaSet, (o, e) => { throw new Exception($"Invalid data: {e.Message}"); });
        return true;
    }
}
